<?php
/** @noinspection PhpFullyQualifiedNameUsageInspection */
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Doctrine\DBAL\Exception as DBALException;
use JsonException;
use RuntimeException;

use function define;
use function defined;
use function file_exists;
use function file_get_contents;
use function json_decode;
use function preg_match;
use function realpath;
use function sprintf;

/**
 * Class WordPressTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 *
 * @runTestsInSeparateProcesses
 */
abstract class WordPressTestCase extends ApplicationTestCase
{
    use DatabaseTestTrait;

    private array $providers = [
        \Inti\Db\ConfigProvider::class,
        \Inti\Wp\Middleware\ConfigProvider::class,
    ];

    /**
     * @throws JsonException
     */
    public static function setUpBeforeClass(): void
    {
        parent::setUpBeforeClass();

        self::defineABSPATH();
        self::checkWordPressIsInstalled();
    }

    /**
     * @throws JsonException
     */
    private static function defineABSPATH(): void
    {
        if (defined('ABSPATH')) {
            return;
        }

        $projectRoot = preg_match('#^(.+)/vendor/inti/test/src/TestCase$#', __DIR__, $matches)
            ? realpath($matches[1])
            : realpath(__DIR__ . '/../..');

        $composerJson = $projectRoot . '/composer.json';
        $composerJson = file_get_contents($composerJson);
        $composerJson = json_decode($composerJson, true, 512, JSON_THROW_ON_ERROR);

        $wordpressInstallDir = $composerJson['extra']['wordpress-install-dir'] ?? 'wordpress';
        $wordpressInstallDir = $projectRoot . '/' . $wordpressInstallDir;

        if (!is_dir($wordpressInstallDir)) {
            throw new RuntimeException(sprintf(
                "%s: WordPress installation directory '%s' does not exist; Maybe run 'composer install'?",
                __METHOD__, $wordpressInstallDir
            ));
        }

        if (!file_exists($wordpressInstallDir . '/wp-includes/plugin.php')) {
            throw new RuntimeException(sprintf(
                "%s: Directory '%s' does not contain a WordPress installation; Maybe run 'composer install'?",
                __METHOD__, $wordpressInstallDir
            ));
        }

        define('ABSPATH', $wordpressInstallDir . '/');
    }

    private static function checkWordPressIsInstalled(): void
    {
        $conn = (new static())->createConnection();

        try {
            $conn->executeQuery('SELECT 1 FROM wp_options');
            return;
        } catch (DBALException $e) {
            throw new RuntimeException(sprintf(
                "%s: WordPress not installed.",
                __METHOD__
            ));
        }
    }
}
